@@ -1,9 +1,12 @@
 """Contains the main display widget used for representing an entire device."""
 
 import enum
+import inspect
 import logging
 import os
 import pathlib
+import webbrowser
+from typing import List, Optional, Union
 
 import ophyd
 import pcdsutils
@@ -16,7 +19,9 @@ from qtpy.QtCore import Q_ENUMS, Property, Qt, Slot
 
 from . import cache
 from . import panel as typhos_panel
-from . import utils, widgets
+from . import utils, web, widgets
+from .jira import TyphosJiraIssueWidget
+from .plugins.core import register_signal
 
 logger = logging.getLogger(__name__)
 
@@ -32,6 +37,19 @@ class DisplayTypes(enum.IntEnum):
 _DisplayTypes = utils.pyqt_class_from_enum(DisplayTypes)
 DisplayTypes.names = [view.name for view in DisplayTypes]
 
+
+class ScrollOptions(enum.IntEnum):
+    """Enumeration of scrollable options for displays."""
+
+    auto = 0
+    scrollbar = 1
+    no_scroll = 2
+
+
+_ScrollOptions = utils.pyqt_class_from_enum(ScrollOptions)
+ScrollOptions.names = [view.name for view in ScrollOptions]
+
+
 DEFAULT_TEMPLATES = {
     name: [(utils.ui_dir / 'core' / f'{name}.ui').resolve()]
     for name in DisplayTypes.names
@@ -44,7 +62,9 @@ DEFAULT_TEMPLATES_FLATTEN = [f for _, files in DEFAULT_TEMPLATES.items()
                              for f in files]
 
 
-def normalize_display_type(display_type):
+def normalize_display_type(
+    display_type: Union[DisplayTypes, str, int]
+) -> DisplayTypes:
     """
     Normalize a given display type.
 
@@ -65,10 +85,45 @@ def normalize_display_type(display_type):
     """
     try:
         return DisplayTypes(display_type)
-    except Exception as ex:
-        if display_type in DisplayTypes.names:
-            return getattr(DisplayTypes, display_type)
-        raise ValueError(f'Unrecognized display type: {display_type}') from ex
+    except ValueError:
+        try:
+            return DisplayTypes[display_type]
+        except KeyError:
+            raise ValueError(
+                f'Unrecognized display type: {display_type}'
+            )
+
+
+def normalize_scroll_option(
+    scroll_option: Union[ScrollOptions, str, int]
+) -> ScrollOptions:
+    """
+    Normalize a given scroll option.
+
+    Parameters
+    ----------
+    display_type : ScrollOptions, str, or int
+        The display type.
+
+    Returns
+    -------
+    display_type : ScrollOptions
+        The normalized :class:`ScrollOptions`.
+
+    Raises
+    ------
+    ValueError
+        If the input cannot be made a :class:`ScrollOptions`.
+    """
+    try:
+        return ScrollOptions(scroll_option)
+    except ValueError:
+        try:
+            return ScrollOptions[scroll_option]
+        except KeyError:
+            raise ValueError(
+                f'Unrecognized scroll option: {scroll_option}'
+            )
 
 
 class TyphosToolButton(QtWidgets.QToolButton):
@@ -305,13 +360,13 @@ class TyphosDisplayConfigButton(TyphosToolButton):
         if not display:
             return base_menu
 
+        base_menu.addSection('Templates')
+        display._generate_template_menu(base_menu)
+
         panels = display.findChildren(typhos_panel.TyphosSignalPanel) or []
         if not panels:
             return base_menu
 
-        base_menu.addSection('Templates')
-        display._generate_template_menu(base_menu)
-
         base_menu.addSection('Filters')
         filter_menu = base_menu.addMenu("&Kind filter")
         self.create_kind_filter_menu(panels, filter_menu, only=False)
@@ -401,8 +456,12 @@ class TyphosDisplaySwitcher(QtWidgets.QFrame, widgets.TyphosDesignerMixin):
     def _create_ui(self):
         layout = self.layout()
         self.buttons.clear()
+        self.help_button = None
         self.config_button = None
 
+        self.help_toggle_button = TyphosHelpToggleButton()
+        layout.addWidget(self.help_toggle_button, 0, Qt.AlignRight)
+
         for template_type in DisplayTypes.names:
             button = TyphosDisplaySwitcherButton(template_type)
             self.buttons[template_type] = button
@@ -457,6 +516,252 @@ class TyphosTitleLabel(QtWidgets.QLabel):
         super().mousePressEvent(event)
 
 
+class TyphosHelpToggleButton(TyphosToolButton):
+    """
+    A standard button used to toggle help information display.
+
+    Attributes
+    ----------
+    pop_out : QtCore.Signal
+        A Qt signal indicating a request to pop out the help widget.
+
+    open_in_browser : QtCore.Signal
+        A Qt signal indicating a request to open the help in a browser.
+
+    open_python_docs : QtCore.Signal
+        A Qt signal indicating a request to open the Python docstring
+        information.
+
+    report_jira_issue : QtCore.Signal
+        A Qt signal indicating a request to open the Jira issue reporting
+        widget.
+
+    toggle_help : QtCore.Signal
+        A Qt signal indicating a request to toggle the related help display
+        frame.
+    """
+    pop_out = QtCore.Signal()
+    open_in_browser = QtCore.Signal()
+    open_python_docs = QtCore.Signal()
+    report_jira_issue = QtCore.Signal()
+    toggle_help = QtCore.Signal(bool)
+
+    def __init__(self, icon="question", parent=None):
+        super().__init__(icon, parent=parent)
+        self.setCheckable(True)
+
+    def _clicked(self):
+        """Hook for QToolButton.clicked."""
+        self.toggle_help.emit(self.isChecked())
+
+    def generate_context_menu(self):
+        menu = QtWidgets.QMenu(parent=self)
+        open_in_browser = menu.addAction("Pop &out documentation...")
+        open_in_browser.triggered.connect(self.pop_out.emit)
+
+        open_in_browser = menu.addAction("Open in &browser...")
+        open_in_browser.triggered.connect(self.open_in_browser.emit)
+
+        open_python_docs = menu.addAction("Open &Python docs...")
+        open_python_docs.triggered.connect(self.open_python_docs.emit)
+
+        def toggle():
+            self.setChecked(not self.isChecked())
+            self._clicked()
+
+        toggle_help = menu.addAction("Toggle &help")
+        toggle_help.triggered.connect(toggle)
+
+        if utils.JIRA_URL:
+            menu.addSeparator()
+            report_issue = menu.addAction("&Report Jira issue...")
+            report_issue.triggered.connect(self.report_jira_issue.emit)
+
+        return menu
+
+
+class TyphosHelpFrame(QtWidgets.QFrame, widgets.TyphosDesignerMixin):
+    """
+    A frame for help information display.
+
+    Attributes
+    ----------
+    tooltip_updated : QtCore.Signal
+        A signal indicating the help tooltip has changed.
+    """
+    tooltip_updated = QtCore.Signal(str)
+
+    def __init__(self, parent=None):
+        super().__init__(parent=parent)
+        self.help = None
+        self.help_web_view = None
+        self._delete_timer = None
+        self.python_docs_browser = None
+
+        self.setContentsMargins(0, 0, 0, 0)
+
+        layout = QtWidgets.QVBoxLayout()
+        self.setLayout(layout)
+        self.devices = []
+        self._jira_widget = None
+
+    def new_jira_widget(self):
+        """Open a new Jira issue reporting widget."""
+        device = self.devices[0] if self.devices else None
+        self._jira_widget = TyphosJiraIssueWidget(device=device)
+        self._jira_widget.show()
+
+    def open_in_browser(self, new=0, autoraise=True):
+        """
+        Open the associated help documentation in the browser.
+
+        Parameters
+        ----------
+        new : int, optional
+            0: the same browser window (the default).
+            1: a new browser window.
+            2: a new browser page ("tab").
+
+        autoraise : bool, optional
+            If possible, autoraise raises the window (the default) or not.
+        """
+        return webbrowser.open(
+            self.help_url.toString(), new=new, autoraise=autoraise
+        )
+
+    def open_python_docs(self):
+        """Open the Python docstring information in a new window."""
+        if self.python_docs_browser is not None:
+            self.python_docs_browser.raise_()
+            return
+
+        self.python_docs_browser = QtWidgets.QTextBrowser()
+        help_document = QtGui.QTextDocument()
+        contents = self._tooltip or "Unset"
+        first_line = contents.splitlines()[0]
+        # TODO: later versions of qt will support setMarkdown
+        help_document.setPlainText(contents)
+        self.python_docs_browser.setWindowTitle(first_line)
+        font = QtGui.QFont("Monospace")
+        font.setStyleHint(QtGui.QFont.TypeWriter)
+        # font.setStyleHint(QtGui.QFont.Monospace)
+        self.python_docs_browser.setFont(font)
+        self.python_docs_browser.setDocument(help_document)
+        self.python_docs_browser.show()
+        return self.python_docs_browser
+
+    def _get_tooltip(self):
+        """Update the tooltip based on device information."""
+        tooltip = []
+        # BUG: I'm seeing two devices in `self.devices` for
+        # $ typhos --fake-device 'ophyd.EpicsMotor[{"prefix":"b"}]'
+        for device in sorted(
+            set(self.devices),
+            key=lambda dev: self.devices.index(dev)
+        ):
+            heading = device.name or type(device).__name__
+            tooltip.extend([
+                heading,
+                "-" * len(heading),
+                ""
+            ])
+
+            tooltip.append(
+                inspect.getdoc(device) or
+                inspect.getdoc(type(device)) or
+                "No docstring"
+            )
+            tooltip.append("")
+
+        return "\n".join(tooltip)
+
+    def add_device(self, device):
+        self.devices.append(device)
+
+        self._tooltip = self._get_tooltip()
+        self.tooltip_updated.emit(self._tooltip)
+
+        self.setWindowTitle(f"Help: {device.name}")
+
+    @property
+    def help_url(self):
+        """The full help URL, generated from ``TYPHOS_HELP_URL``."""
+        if not self.devices:
+            return QtCore.QUrl("about:blank")
+
+        device, *_ = self.devices
+        try:
+            device_url = utils.HELP_URL.format(device=device)
+        except Exception:
+            logger.exception("Failed to format confluence URL for device %s",
+                             device)
+            return QtCore.QUrl("about:blank")
+
+        return QtCore.QUrl(device_url)
+
+    def show_help(self):
+        """Show the help information in a QWebEngineView."""
+        if web.TyphosWebEngineView is None:
+            logger.error(
+                "Failed to import QWebEngineView; "
+                "help view is unavailable."
+                )
+            return
+
+        if self.help_web_view:
+            self.help_web_view.show()
+            return
+
+        self.help_web_view = web.TyphosWebEngineView()
+        self.help_web_view.page().setUrl(self.help_url)
+
+        self.help_web_view.setEnabled(True)
+        self.help_web_view.setMinimumSize(QtCore.QSize(100, 400))
+
+        self.layout().addWidget(self.help_web_view)
+
+    def hide_help(self):
+        """Hide the help information QWebEngineView."""
+        if not self.help_web_view:
+            return
+        self.help_web_view.hide()
+        if self._delete_timer is None:
+            self._delete_timer = QtCore.QTimer()
+            self._delete_timer.setInterval(20000)
+            self._delete_timer.setSingleShot(True)
+            self._delete_timer.timeout.connect(self._delete_help_if_hidden)
+            self._delete_timer.start()
+
+    def _delete_help_if_hidden(self):
+        """
+        Slowly react to the help display removal, as setting it back up can be
+        slow and painful.
+        """
+        self._delete_timer = None
+        if self.help_web_view and not self.help_web_view.isVisible():
+            self.layout().removeWidget(self.help_web_view)
+            self.help_web_view.deleteLater()
+            self.help_web_view = None
+
+    def toggle_help(self, show):
+        """
+        Toggle the visibility of the help information QWebEngineView.
+
+        Parameters
+        ----------
+        show : bool
+            Show the help (True) or hide it (False).
+        """
+        if not self.devices:
+            logger.warning("No devices added -> no help")
+            return
+
+        if show:
+            self.show_help()
+        else:
+            self.hide_help()
+
+
 class TyphosDisplayTitle(QtWidgets.QFrame, widgets.TyphosDesignerMixin):
     """
     Standardized Typhos Device Display title.
@@ -494,6 +799,34 @@ class TyphosDisplayTitle(QtWidgets.QFrame, widgets.TyphosDesignerMixin):
         self.grid_layout.addWidget(self.label, 0, 0)
         self.grid_layout.addWidget(self.switcher, 0, 1, Qt.AlignRight)
         self.grid_layout.addWidget(self.underline, 1, 0, 1, 2)
+
+        if not utils.HELP_URL:
+            # The help widget is entirely optional, based on environment
+            # settings.
+            self.help = None
+        else:
+            self.help = TyphosHelpFrame()
+            self.switcher.help_toggle_button.toggle_help.connect(
+                self.toggle_help
+            )
+            self.switcher.help_toggle_button.pop_out.connect(
+                self.pop_out_help
+            )
+            self.switcher.help_toggle_button.open_in_browser.connect(
+                self.help.open_in_browser
+            )
+            self.switcher.help_toggle_button.open_python_docs.connect(
+                self.help.open_python_docs
+            )
+            self.switcher.help_toggle_button.report_jira_issue.connect(
+                self.help.new_jira_widget
+            )
+            self.help.tooltip_updated.connect(
+                self.switcher.help_toggle_button.setToolTip
+            )
+
+            self.grid_layout.addWidget(self.help, 2, 0, 1, 2)
+
         self.grid_layout.setSizeConstraint(self.grid_layout.SetMinimumSize)
         self.setLayout(self.grid_layout)
 
@@ -501,6 +834,26 @@ class TyphosDisplayTitle(QtWidgets.QFrame, widgets.TyphosDesignerMixin):
         self.show_switcher = show_switcher
         self.show_underline = show_underline
 
+    def toggle_help(self, show):
+        """Toggle the help visibility."""
+        if self.help is None:
+            return
+
+        self.help.toggle_help(show)
+        if self.help.parent() is None:
+            self.grid_layout.addWidget(self.help, 2, 0, 1, 2)
+
+    def pop_out_help(self):
+        """Pop out the help widget."""
+        if self.help is None:
+            return
+
+        self.help.setParent(None)
+        self.switcher.help_toggle_button.setChecked(True)
+        self.help.show_help()
+        self.help.show()
+        self.help.raise_()
+
     @Property(bool)
     def show_switcher(self):
         """Get or set whether to show the display switcher."""
@@ -516,6 +869,9 @@ class TyphosDisplayTitle(QtWidgets.QFrame, widgets.TyphosDesignerMixin):
         if not self.label.text():
             self.label.setText(device.name)
 
+        if self.help is not None:
+            self.help.add_device(device)
+
     @QtCore.Property(bool)
     def show_underline(self):
         """Get or set whether to show the underline."""
@@ -583,8 +939,11 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
         The parent widget.
 
     scrollable : bool, optional
+        Semi-deprecated parameter. Use scroll_option instead.
         If ``True``, put the loaded template into a :class:`QScrollArea`.
-        Otherwise, the display widget will go directly in this widget's layout.
+        If ``False``, the display widget will go directly in this widget's
+        layout.
+        If omitted, scroll_option is used instead.
 
     composite_heuristics : bool, optional
         Enable composite heuristics, which may change the suggested detailed
@@ -601,9 +960,12 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
         List of engineering templates to use in addition to those found on
         disk.
 
-    display_type : DisplayTypes, optional
+    display_type : DisplayTypes, str, or int, optional
         The default display type.
 
+    scroll_option : ScrollOptions, str, or int, optional
+        The scroll behavior.
+
     nested : bool, optional
         An optional annotation for a display that may be nested inside another.
     """
@@ -615,17 +977,25 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
     device_count_threshold = 0
     signal_count_threshold = 30
 
-    def __init__(self, parent=None, *, scrollable=True,
-                 composite_heuristics=True, embedded_templates=None,
-                 detailed_templates=None, engineering_templates=None,
-                 display_type='detailed_screen', nested=False):
-
+    def __init__(
+        self,
+        parent: Optional[QtWidgets.QWidget] = None,
+        *,
+        scrollable: Optional[bool] = None,
+        composite_heuristics: bool = True,
+        embedded_templates: Optional[List[str]] = None,
+        detailed_templates: Optional[List[str]] = None,
+        engineering_templates: Optional[List[str]] = None,
+        display_type: Union[DisplayTypes, str, int] = 'detailed_screen',
+        scroll_option: Union[ScrollOptions, str, int] = 'auto',
+        nested: bool = False,
+    ):
         self._composite_heuristics = composite_heuristics
         self._current_template = None
         self._forced_template = ''
         self._macros = {}
         self._display_widget = None
-        self._scrollable = False
+        self._scroll_option = ScrollOptions.no_scroll
         self._searched = False
         self._hide_empty = False
         self._nested = nested
@@ -657,7 +1027,13 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
         layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(self._scroll_area)
 
-        self.scrollable = scrollable
+        if scrollable is None:
+            self.scroll_option = scroll_option
+        else:
+            if scrollable:
+                self.scroll_option = ScrollOptions.scrollbar
+            else:
+                self.scroll_option = ScrollOptions.no_scroll
 
     @Property(bool)
     def composite_heuristics(self):
@@ -668,18 +1044,19 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
     def composite_heuristics(self, composite_heuristics):
         self._composite_heuristics = bool(composite_heuristics)
 
-    @Property(bool)
-    def scrollable(self):
+    @Property(_ScrollOptions)
+    def scroll_option(self) -> ScrollOptions:
         """Place the display in a scrollable area."""
-        return self._scrollable
+        return self._scroll_option
 
-    @scrollable.setter
-    def scrollable(self, scrollable):
-        # Switch between using the scroll area layout or
-        if scrollable == self._scrollable:
+    @scroll_option.setter
+    def scroll_option(self, scrollable: ScrollOptions):
+        # Switch the scroll area behavior
+        opt = normalize_scroll_option(scrollable)
+        if opt == self._scroll_option:
             return
 
-        self._scrollable = bool(scrollable)
+        self._scroll_option = opt
         self._move_display_to_layout(self._display_widget)
 
     @Property(bool)
@@ -697,12 +1074,24 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
             return
 
         widget.setParent(None)
-        if self._scrollable:
+        if self.scroll_option == ScrollOptions.auto:
+            if self.display_type == DisplayTypes.embedded_screen:
+                scrollable = False
+            else:
+                scrollable = True
+        elif self.scroll_option == ScrollOptions.scrollbar:
+            scrollable = True
+        elif self.scroll_option == ScrollOptions.no_scroll:
+            scrollable = False
+        else:
+            scrollable = True
+
+        if scrollable:
             self._scroll_area.setWidget(widget)
         else:
             self.layout().addWidget(widget)
 
-        self._scroll_area.setVisible(self._scrollable)
+        self._scroll_area.setVisible(scrollable)
 
     def _generate_template_menu(self, base_menu):
         """Generate the template switcher menu, adding it to ``base_menu``."""
@@ -974,6 +1363,10 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
            3. The argument ``macros`` is then used to fill/update the final
               macro dictionary.
 
+        This will also register the device's signals in the sig:// plugin.
+        This means that any templates can refer to their device's signals by
+        name.
+
         Parameters
         ----------
         device : ophyd.Device
@@ -988,6 +1381,9 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
             self.devices.clear()
         # Add the device to the cache
         super().add_device(device)
+        logger.debug("Registering signals from device %s", device.name)
+        for component_walk in device.walk_signals():
+            register_signal(component_walk.item)
         self._searched = False
         self.macros = self._build_macros_from_device(device, macros=macros)
         self.load_best_template()
@@ -1133,7 +1529,7 @@ class TyphosDeviceDisplay(utils.TyphosBase, widgets.TyphosDesignerMixin,
             obj = pcdsutils.utils.get_instance_by_name(klass, **kwargs)
         except Exception:
             logger.exception('Failed to generate TyphosDeviceDisplay from '
-                             'device %s', obj)
+                             'class %s', klass)
             return None
 
         return cls.from_device(obj, template=template, macros=macros)
