@@ -6,6 +6,7 @@ from functools import wraps
 
 import numpy as np
 import ophyd.sim
+import pydm
 import pytest
 import qtpy
 from happi import Client
@@ -14,10 +15,12 @@ from ophyd import Device
 from ophyd import FormattedComponent as FC
 from ophyd.sim import Signal, SynAxis, SynPeriodicSignal
 from pydm import PyDMApplication
+from pydm.data_plugins import plugin_for_address
 from pydm.widgets.logdisplay import GuiHandler
 from qtpy import QtGui, QtWidgets
 
 import typhos
+from typhos.plugins.core import signal_registry
 from typhos.plugins.happi import register_client
 from typhos.utils import SignalRO, TyphosBase
 
@@ -54,14 +57,22 @@ def _show_widgets(pytestconfig):
 @pytest.fixture(scope='session', autouse=True)
 def qapp(pytestconfig):
     global application
-    if application:
-        pass
-    else:
+    application = QtWidgets.QApplication.instance()
+    if application is None:
         application = PyDMApplication(use_main_window=False)
-        typhos.use_stylesheet(pytestconfig.getoption('--dark'))
+    typhos.use_stylesheet(pytestconfig.getoption('--dark'))
     return application
 
 
+@pytest.fixture(scope='function', autouse=True)
+def noapp(monkeypatch):
+    monkeypatch.setattr(QtWidgets.QApplication, 'exec_', lambda x: 1)
+    monkeypatch.setattr(QtWidgets.QApplication, 'exit', lambda x: 1)
+    monkeypatch.setattr(
+        pydm.exception, 'raise_to_operator', lambda *_, **__: None
+    )
+
+
 @pytest.fixture(scope='session')
 def test_images():
     return (os.path.join(os.path.dirname(__file__), 'utils/lenna.png'),
@@ -219,7 +230,7 @@ class MockDevice(Device):
 
 @pytest.fixture(scope='function')
 def device():
-    dev = MockDevice('Tst:This', name='Simulated Device')
+    dev = MockDevice('Tst:This', name='simulated_device')
     yield dev
     clear_handlers(dev)
 
@@ -248,3 +259,22 @@ def happi_cfg():
     path = str(MODULE_PATH / 'happi.cfg')
     os.environ['HAPPI_CFG'] = path
     return path
+
+
+@pytest.fixture(scope='function', autouse=True)
+def reset_signal_plugin():
+    """
+    Completely restart the sig:// plugin.
+
+    After the restart, there will be no open SignalConnection objects
+    and nothing in the signal registry.
+
+    Some tests are easier to express by repeating signal names, which
+    will cause the signal plugin to ignore the new devices in favor of
+    the previously saved devices, which are no longer available to be
+    manipulated or tested.
+    """
+    signal_registry.clear()
+    plugin = plugin_for_address('sig://test')
+    for channel in list(plugin.channels):
+        plugin.remove_connection(channel)
