@@ -1,10 +1,8 @@
 import os
 
-import pydm
 import pytest
 
 import typhos
-from typhos.app import QApplication
 from typhos.cli import typhos_cli
 
 from . import conftest
@@ -16,8 +14,7 @@ def test_cli_version(capsys):
     assert typhos.__version__ in readout.out
 
 
-def test_cli_happi_cfg(monkeypatch, qtbot, happi_cfg):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
+def test_cli_happi_cfg(qtbot, happi_cfg):
     window = typhos_cli(['test_motor', '--happi-cfg', happi_cfg])
     qtbot.addWidget(window)
     assert window.isVisible()
@@ -29,19 +26,14 @@ def test_cli_bad_entry(qtbot, happi_cfg):
     assert window is None
 
 
-def test_cli_no_entry(monkeypatch, qtbot, happi_cfg):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
+def test_cli_no_entry(qtbot, happi_cfg):
     window = typhos_cli(['--happi-cfg', happi_cfg])
     qtbot.addWidget(window)
     assert window.isVisible()
     assert window.centralWidget().devices == []
 
 
-def test_cli_stylesheet(monkeypatch, qapp, qtbot, happi_cfg):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
-    monkeypatch.setattr(
-        pydm.exception, 'raise_to_operator', lambda *_, **__: None
-    )
+def test_cli_stylesheet(qapp, qtbot, happi_cfg):
     with open('test.qss', 'w+') as handle:
         handle.write(
             "TyphosDeviceDisplay {qproperty-force_template: 'test.ui'}")
@@ -60,8 +52,7 @@ def test_cli_stylesheet(monkeypatch, qapp, qtbot, happi_cfg):
     ("ophyd.sim.SynAxis[]", "SynAxis"),
     ("ophyd.sim.SynAxis[{'name':'foo'}]", "foo")
 ])
-def test_cli_class(monkeypatch, qapp, qtbot, klass, name, happi_cfg):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
+def test_cli_class(qtbot, klass, name, happi_cfg):
     window = typhos_cli([klass])
     qtbot.addWidget(window)
     assert window.isVisible()
@@ -78,8 +69,7 @@ def test_cli_class_invalid(qtbot):
     assert window is None
 
 
-def test_cli_profile_modules(monkeypatch, capsys, qapp, qtbot):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
+def test_cli_profile_modules(capsys, qtbot):
     window = typhos_cli(['ophyd.sim.SynAxis[]', '--profile-modules',
                          'typhos.suite'])
     qtbot.addWidget(window)
@@ -87,9 +77,7 @@ def test_cli_profile_modules(monkeypatch, capsys, qapp, qtbot):
     assert 'add_device' in output.out
 
 
-def test_cli_benchmark(monkeypatch, capsys, qapp, qtbot):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
-    monkeypatch.setattr(QApplication, 'exit', lambda x: 1)
+def test_cli_benchmark(capsys, qtbot):
     windows = typhos_cli(['ophyd.sim.SynAxis[]', '--benchmark',
                           'flat_soft'])
     qtbot.addWidget(windows[0])
@@ -97,8 +85,7 @@ def test_cli_benchmark(monkeypatch, capsys, qapp, qtbot):
     assert 'add_device' in output.out
 
 
-def test_cli_profile_output(monkeypatch, capsys, qapp, qtbot):
-    monkeypatch.setattr(QApplication, 'exec_', lambda x: 1)
+def test_cli_profile_output(capsys, qtbot):
     path_obj = conftest.MODULE_PATH / 'artifacts' / 'prof'
     if not path_obj.parent.exists():
         path_obj.parent.mkdir(parents=True)
