@@ -16,7 +16,7 @@ import pydm.widgets.enum_button
 import qtawesome as qta
 from ophyd.signal import EpicsSignalBase
 from pydm.widgets.display_format import DisplayFormat
-from pyqtgraph.parametertree import parameterTypes as ptypes
+from pyqtgraph.parametertree import ParameterItem
 from qtpy import QtGui, QtWidgets
 from qtpy.QtCore import Property, QObject, QSize, Qt, Signal, Slot
 from qtpy.QtWidgets import (QAction, QDialog, QDockWidget, QPushButton,
@@ -157,6 +157,41 @@ class TyphosComboBox(pydm.widgets.PyDMEnumComboBox):
     Notes
     -----
     """
+    def __init__(self, *args, variety_metadata=None, ophyd_signal=None,
+                 **kwargs):
+        super().__init__(*args, **kwargs)
+        self.ophyd_signal = ophyd_signal
+        self._ophyd_enum_strings = None
+        self._md_sub = ophyd_signal.subscribe(
+            self._metadata_update, event_type="meta"
+        )
+
+    def __dtor__(self):
+        """PyQt5 destructor hook."""
+        if self._md_sub is not None:
+            self.ophyd_signal.unsubscribe(self._md_sub)
+            self._md_sub = None
+
+    def _metadata_update(self, enum_strs=None, **kwargs):
+        if enum_strs:
+            self._ophyd_enum_strings = tuple(enum_strs)
+            self.enum_strings_changed(enum_strs)
+
+    def enum_strings_changed(self, new_enum_strings):
+        current_idx = self.currentIndex()
+        super().enum_strings_changed(
+            tuple(self._ophyd_enum_strings or new_enum_strings)
+        )
+        self.value_changed(current_idx)
+
+    def wheelEvent(self, event: QtGui.QWheelEvent):
+        event.ignore()
+
+
+class NoScrollComboBox(QtWidgets.QComboBox):
+    """
+    A combobox disconnected from direct EPICS/ophyd with scrolling ignored.
+    """
     def wheelEvent(self, event: QtGui.QWheelEvent):
         event.ignore()
 
@@ -306,7 +341,9 @@ class TyphosLabel(pydm.widgets.PyDMLabel):
     Notes
     -----
     """
-    def __init__(self, *args, display_format=None, **kwargs):
+    def __init__(
+        self, *args, display_format=None, ophyd_signal=None, **kwargs
+    ):
         super().__init__(*args, **kwargs)
         self.setAlignment(Qt.AlignCenter)
         self.setSizePolicy(QtWidgets.QSizePolicy.Preferred,
@@ -315,6 +352,28 @@ class TyphosLabel(pydm.widgets.PyDMLabel):
         if display_format is not None:
             self.displayFormat = display_format
 
+        self.ophyd_signal = ophyd_signal
+        self._ophyd_enum_strings = None
+        self._md_sub = ophyd_signal.subscribe(
+            self._metadata_update, event_type="meta"
+        )
+
+    def __dtor__(self):
+        """PyQt5 destructor hook."""
+        if self._md_sub is not None:
+            self.ophyd_signal.unsubscribe(self._md_sub)
+            self._md_sub = None
+
+    def _metadata_update(self, enum_strs=None, **kwargs):
+        if enum_strs:
+            self._ophyd_enum_strings = tuple(enum_strs)
+            self.enum_strings_changed(enum_strs)
+
+    def enum_strings_changed(self, new_enum_strings):
+        super().enum_strings_changed(
+            tuple(self._ophyd_enum_strings or new_enum_strings)
+        )
+
     def unit_changed(self, new_unit):
         """
         Callback invoked when the Channel has new unit value.
@@ -335,7 +394,7 @@ class TyphosLabel(pydm.widgets.PyDMLabel):
             self.displayFormat = DisplayFormat.Exponential
 
 
-class TyphosSidebarItem(ptypes.ParameterItem):
+class TyphosSidebarItem(ParameterItem):
     """
     Class to display a Device or Tool in the sidebar
 
@@ -833,7 +892,11 @@ class TyphosScalarRange(pydm.widgets.PyDMSlider):
     def __dtor__(self):
         """PyQt5 destructor hook"""
         # Ensure our delta signal subscription is cleared:
-        self.delta_signal = None
+        if self._delta_signal is not None:
+            if self._delta_signal_sub is not None:
+                self._delta_signal.unsubscribe(self._delta_signal_sub)
+                self._delta_signal_sub = None
+            self.delta_signal = None
 
     @variety.key_handler('range')
     def _variety_key_handler_range(self, value, source, **kwargs):
@@ -880,6 +943,7 @@ class TyphosScalarRange(pydm.widgets.PyDMSlider):
     def delta_signal(self, signal):
         if self._delta_signal is not None:
             self._delta_signal.unsubscribe(self._delta_signal_sub)
+            self._delta_signal_sub = None
 
         if signal is None:
             return
@@ -1042,6 +1106,9 @@ def _get_ndimensional_widget_class(dimensions, desc, variety_md, read_only):
     }.get(dimensions, TyphosLabel)
 
 
+DIRECT_CONTROL_LAYERS = {"pyepics", "caproto"}
+
+
 def widget_type_from_description(signal, desc, read_only=False):
     """
     Determine which widget class should be used for the given signal
@@ -1064,7 +1131,14 @@ def widget_type_from_description(signal, desc, read_only=False):
     kwargs : dict
         Keyword arguments for the class
     """
-    if isinstance(signal, EpicsSignalBase):
+    use_pv_directly = (
+        # We can use PyDM's data source directly with EpicsSignalBase:
+        isinstance(signal, EpicsSignalBase) and
+        # So long as its underlying control layer is a supported ophyd-provided
+        # one, at least.
+        getattr(signal.cl, "name", "") in DIRECT_CONTROL_LAYERS
+    )
+    if use_pv_directly:
         # Still re-route EpicsSignal through the ca:// plugin
         pv = (signal._read_pv
               if read_only else signal._write_pv)
